"""
Threading prototypes
"""

# 	Copyright (c) 2005-2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import types
from threading import Thread
from cxutil.utils import opts
from cxutil.UID import UID
from logger import log
import traceback



def passive(glist):
	def f(func):
		glist.append(func.func_name)
		return func
	return f

def push(glist):
	def f(func):
		glist[func.func_name] = None
		return func
	return f

class Generator (log):
	'''
	Generator function wrapper
	'''
	def __init__(self,func,a,bus):
		self.uid = UID()
		self.queue = []
		self.a = a
		self.func = func
		self.bus = bus
		self.queue.append(self.func.next())
	
	def uid(self):
		return self.uid
	
	def next(self):
		try:
			self.queue.append(self.func.next())
		except StopIteration:
			# commit suicide
			self.log("debug","close generator") #, target: `%s`, message: `%s`" % (target,message))
			del self.a[self.uid]
		except:
			raise
		
		return self.queue.pop(0)

class ManagerThread (opts,log,Thread):
	'''
	A manager object, an event handler for dispatcher2
	'''
	bus = None
	generators = {}
	passive_generators = []

	def __init__(self,bus,name,dct={}):
		opts.__init__(self,dct)
		Thread.__init__(self)

		self.setName(name)
		self.bus = bus
		self.passive_queue = []

		if hasattr(self,"post"):
			self.post()


	def __run_generator(self,target,message):
		if not self.generators[target]:
			self.generators[target] = {}
		
		result = None
		stop = False

		# >= 2 call
		if message.has_key("generator_cookie"):

			cookie = message["generator_cookie"]

			if cookie in self.generators[target].keys():
				self.log("debug","call for next generator value, target: `%s`, message: `%s`" % (target,message))
				try:
					result = self.generators[target][cookie].next()
				except:
					self.log("error",traceback.format_exc())

			else:
				self.log("warning","call for non-existent cookie value, target: `%s`, message: `%s`" % (target,message))
				return

		# 1st call
		else:
			generator = getattr(self,target)(message)
			self.log("debug","create generator, target: `%s`, message: `%s`" % (target,message))
			g = Generator(generator,self.generators[target],self.bus)
			self.generators[target][g.uid()] = g
			return g.uid()

		return result

	def run(self):
		while True:
			addr = {}
			# print "%s: waiting for message" % (self.getName())
			message = self.bus.get(addr)
			# print "%s: got message from %s: %s" % (self.getName(), addr["from"], message)

			if \
				isinstance(message,dict) or\
				isinstance(message,opts):
				message["addr"] = addr["from"]
				target = message["map"]
				if hasattr(self,target):

					func = getattr(self,target)
					
					if target in self.passive_generators:
						###
						#
						# Create and enqueue passive generator
						#
						###
						self.passive_queue.append((addr["from"],func(message)))
						message = None

					elif target in self.generators.keys():
						###
						#
						# Get a message from active generator
						#
						###
						message = self.__run_generator(target,message)
					else:
						###
						#
						# Run ordinary function and return a message
						#
						###
						try:
							message = func(message)
						except:
							self.log("critical",traceback.format_exc())
				else:
					self.log("error","unknown symbol %s" % (target))

				self.send(addr["from"],message)
				###
				#
				# Iterate passive generators
				#
				###
				i = 0
				while True:
					try:
						m = self.passive_queue[i][1].next()
						# run iterator
						self.send(self.passive_queue[i][0],m)
						i += 1
					except StopIteration:
						# if empty, pop it from the list
						self.passive_queue.pop(i)
					except IndexError:
						# end of queue, break the cycle
						break
					except:
						self.log("error","error while running iterator: %s" % (traceback.format_exc()))

			elif message == "shutdown":
				return

	def send(self,addr,message):
		if not isinstance(message,type(None)):
			# print "%s: send message to %s: %s" % (self.getName(), addr, message)
			try:
				if message.has_key("to"):
					addr = message["to"]
			except:
				pass
			self.bus.put(addr,message)
